@@ -9,7 +9,7 @@
 #
 package Lemonldap::NG::Portal::Main::Run;
 
-our $VERSION = '2.20.0';
+our $VERSION = '2.21.0';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -328,6 +328,7 @@ sub unauthLogout {
 
 sub _unauthLogout {
     my ( $self, $req ) = @_;
+    $self->processHook( $req, 'unAuthLogout' );
     $self->logger->debug('Unauthenticated logout request');
     $self->logger->debug('Cleaning pdata');
     $self->logger->debug("Removing $self->{conf}->{cookieName} cookie");
@@ -490,9 +491,10 @@ sub autoRedirect {
         if ( $req->{urldc} =~ /^\s*((?:java|vb)script|data):/ ) {
             $self->auditLog(
                 $req,
-                message => "Redirection to $req->{urldc} blocked",
-                code    => "UNAUTHORIZED_REDIRECT",
-                url     => $req->{urldc},
+                message  => "Redirection to $req->{urldc} blocked",
+                code     => "UNAUTHORIZED_REDIRECT",
+                url      => $req->{urldc},
+                user     => $req->sessionInfo->{ $self->conf->{whatToTrace} } || $req->user,
             );
             delete $req->{urldc};
         }
@@ -791,8 +793,9 @@ sub _deleteSession {
             message => (
 "User $user has been disconnected from $mod ($req->{sessionInfo}->{ipAddr})"
             ),
-            code => "LOGOUT",
-            auth => $mod,
+            code     => "LOGOUT",
+            user     => $user,
+            auth     => $mod,
         );
     }
 
@@ -826,9 +829,10 @@ sub autoPost {
     if ( $req->{urldc} =~ /^\s*((?:java|vb)script|data):/ ) {
         $self->auditLog(
             $req,
-            message => "Redirection to $req->{urldc} blocked",
-            code    => "UNAUTHORIZED_REDIRECT",
-            url     => $req->{urldc},
+            message  => "Redirection to $req->{urldc} blocked",
+            code     => "UNAUTHORIZED_REDIRECT",
+            url      => $req->{urldc},
+            user     => $req->sessionInfo->{ $self->conf->{whatToTrace} } || $req->user,
         );
         return PE_BADURL;
     }
@@ -1062,6 +1066,10 @@ sub getTrOver {
 
 sub sendHtml {
     my ( $self, $req, $template, %args ) = @_;
+    if ( $self->conf->{logParams} ) {
+        $self->_dump($req);
+        $self->_dump( \%args );
+    }
 
     my $skin_template_dir = $self->getSkinTplDir( $self->getSkin($req) );
 
@@ -1277,10 +1285,13 @@ sub tplParams {
     }
 
     return (
+        CACHE_TAG    => $self->cacheTag,
         PORTAL_URL   => $req->portal,
+        PORTAL_BASE  => $portalPath,
         MAIN_LOGO    => $self->conf->{portalMainLogo},
         LANGS        => $self->conf->{showLanguages},
         SCROLL_TOP   => $self->conf->{scrollTop},
+        FLOATING_CAT => $self->conf->{floatingCategoryName},
         SKIN         => $self->getSkin($req),
         SKIN_PATH    => $portalPath . "skins",
         SAMESITE     => getSameSite( $self->conf ),
@@ -1545,10 +1556,25 @@ sub cspGetHost {
     return;
 }
 
+sub relativeUrl {
+    my $self = shift;
+    my $req  = shift;
+
+    my $uri = $self->_buildUrlObj( $req->portal, @_ );
+    return $uri->path_query;
+}
+
 sub buildUrl {
     my $self = shift;
     return $self->portal unless @_;
 
+    return $self->_buildUrlObj(@_)->as_string;
+}
+
+sub _buildUrlObj {
+    my $self = shift;
+    return URI->new( $self->portal ) unless @_;
+
     # URL base is $self->portal unless first arg is an URL
     my $uri =
       URI->new( ( $_[0] =~ m#^https?://# ) ? shift(@_) : $self->portal );
@@ -1567,7 +1593,7 @@ sub buildUrl {
         }
     }
     $uri->path_segments(@pathSg);
-    return $uri->as_string;
+    return $uri;
 }
 
 sub rememberBrowser {
